@@ -170,7 +170,6 @@ Current scheme is to have the total zize of element followed by the type, then t
 void read_history ()
 {
 	size_t x;
-	int is_locked=0;
   /* Build file path */
   gchar* history_path = g_build_filename(g_get_user_data_dir(),HISTORY_FILE0,NULL); 
 	gchar *magic=g_malloc0(2+HISTORY_MAGIC_SIZE);
@@ -197,7 +196,6 @@ void read_history ()
 		if(dbg) g_printf("History Magic OK. Reading\n");
     /* Continue reading until size is 0 */
 		g_mutex_lock(&hist_lock);
-		is_locked=1;
     while (size)   {
 			struct history_item *c;
 			if (fread(&size, 4, 1, history_file) != 1)
@@ -231,8 +229,7 @@ done:
     /* Close file and reverse the history to normal */
     fclose(history_file);
     history_list = g_list_reverse(history_list);
-		if(is_locked)
-			g_mutex_unlock(&hist_lock);
+		g_mutex_unlock(&hist_lock);
   }
 	if(dbg) g_printf("History read done\n");
 }
@@ -390,11 +387,7 @@ void append_item(gchar* item, int checkdup, gint iflags, gint itype)
 	}else{ /**not found   */
 		if(NULL == (c=new_clip_item(CLIP_TYPE_TEXT,strlen(item),item)) )
 			return;
-		if(node > -1 && (checkdup & HIST_KEEP_FLAGS) ){
-			c->flags=flags;
-			/*g_printf("Restoring 0x%02X '%s'\n",c->flags,c->text);  */
-		}	else
-			c->flags=iflags;
+		c->flags=iflags;
 		c->type=itype;
 	}
 	
@@ -513,31 +506,24 @@ void clear_history( void )
 /***************************************************************************/
 /** .
 \n\b Arguments:
-\n\b path - path to file to save
-\n\b mode - 1 to print line numbers.
 \n\b Returns:
 ****************************************************************************/
-int save_history_as_text(gchar *path, int mode)
+int save_history_as_text(gchar *path)
 {
 	FILE* fp = fopen(path, "w");
   /* Check that it opened and begin write */
   if (fp)  {
 		gint i;
     GList* element;
-		gchar *fstring="%s\n";
 		g_mutex_lock(&hist_lock);
     /* Write each element to  file */
     for (i=0,element = history_list; element != NULL; element = element->next) {
 		  struct history_item *c;
 			c=(struct history_item *)element->data;
-			if(mode){
-				if(c->flags & CLIP_TYPE_PERSISTENT)
-					fstring="PHIST_%04d %s\n";
-				else
-					fstring="NHIST_%04d %s\n";	
-				fprintf(fp,fstring,i,c->text);
-			}else
-				fprintf(fp,fstring,c->text);
+			if(c->flags & CLIP_TYPE_PERSISTENT)
+				fprintf(fp,"PHIST_%04d %s\n",i,c->text);
+			else
+				fprintf(fp,"NHIST_%04d %s\n",i,c->text);
 			++i;
     }
 		g_mutex_unlock(&hist_lock);
@@ -572,7 +558,7 @@ void history_save_as(GtkMenuItem *menu_item, gpointer user_data)
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
 	    gchar *filename;
 	    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-	    save_history_as_text (filename, get_pref_int32("save_history_Lineno"));
+	    save_history_as_text (filename);
 	    g_free (filename);
 	  }
 	gtk_widget_destroy (dialog);
