@@ -79,8 +79,8 @@ grab_ungrab_with_ignorable_modifiers (GdkWindow *rootwin,
 		caps_lock_mask | scroll_lock_mask,
 		num_lock_mask  | caps_lock_mask | scroll_lock_mask,
 	};
-	int i;
 
+	size_t i;
 	for (i = 0; i < G_N_ELEMENTS (mod_masks); i++) {
 		if (grab) {
 			XGrabKey (GDK_WINDOW_XDISPLAY (rootwin), 
