@@ -65,9 +65,8 @@ g_signal_connect(clipboard, "owner-change",  G_CALLBACK(handle_owner_change), NU
 #include "parcellite.h"
 
 #include <ctype.h>
-#include <pthread.h>
-#include <libgen.h>
-
+#include <errno.h>
+#include <sys/file.h>
 
 /**ACT are actions, and MODE is the mode of the action  */
 /** #define ACT_STOP  0
@@ -89,7 +88,6 @@ g_signal_connect(clipboard, "owner-change",  G_CALLBACK(handle_owner_change), NU
 #define FIFCMD_RUN_PRI "run_pri"
 #define FIFCMD_RUN_CLI "run_cli"
 #define FIFCMD_RUN_ALL "run_all"
-#define FIFCMD_CLEAR_ALL "clear_all"
 
 GtkWidget *hmenu;
 /**see parcellite.h for DEBUG defines  */
@@ -149,7 +147,7 @@ static struct pref2int pref2int_map[]={
 void create_app_indicator(void);
 
 /**Turns up in 2.16  */
-int p_strcmp (const char *str1, const char *str2)
+static int p_strcmp (const char *str1, const char *str2)
 {
 #if (GTK_MAJOR_VERSION > 2 || ( GTK_MAJOR_VERSION == 2 && GTK_MAJOR_VERSION >= 16))
   return g_strcmp0(str1,str2);
@@ -170,9 +168,9 @@ WARNING! This modifies ntext!
 \n\b Arguments:
 \n\b Returns: processed text, or NULL if it is invalid.
 ****************************************************************************/
-gchar *process_new_item(GtkClipboard *clip,gchar *ntext, int *mod)
+static gchar *process_new_item(GtkClipboard *clip,gchar *ntext, int *mod)
 {
-	glong len,nlen;
+	size_t len, nlen;
 	gchar *rtn=NULL;
 	int i=0;
 	if(NULL != mod)
@@ -244,7 +242,7 @@ done:
 \n\b Arguments:
 \n\b Returns:	text that was updated or NULL if not.
 ****************************************************************************/
-gchar *_update_clipboard (GtkClipboard *clip, gchar *n, gchar **old, int set, int mode)
+static gchar *_update_clipboard (GtkClipboard *clip, gchar *n, gchar **old, int set, int mode)
 {
 	
 	/*return NULL; */
@@ -277,7 +275,7 @@ gchar *_update_clipboard (GtkClipboard *clip, gchar *n, gchar **old, int set, in
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean content_exists(GtkClipboard *clip)
+static gboolean content_exists(GtkClipboard *clip)
 {
 	gint count;
 	GdkAtom *targets;
@@ -291,7 +289,7 @@ gboolean content_exists(GtkClipboard *clip)
 \n\b Arguments:
 \n\b Returns:	 clipboard contents, to be freed with g_free, or NULL if empty.
 ****************************************************************************/
-gchar *is_clipboard_empty(GtkClipboard *clip)
+static gchar *is_clipboard_empty(GtkClipboard *clip)
 {
   /** int count;
   GdkAtom *targets;
@@ -331,7 +329,7 @@ intext is the text coming from the clip (above)
 processed is set if we handle this clip with the options we have
 \n\b Returns: set if we changed the new text or not.
 ****************************************************************************/
-int do_we_set_clipboard(GtkClipboard *clip,gchar *intext,gchar **processed, int *mod)
+static int do_we_set_clipboard(GtkClipboard *clip,gchar *intext,gchar **processed, int *mod)
 {
 	gchar *unproc_clip;
 	int set;
@@ -354,7 +352,7 @@ int do_we_set_clipboard(GtkClipboard *clip,gchar *intext,gchar **processed, int
 \n\b Arguments:
 \n\b Returns: text that was placed in history or NULL
 ****************************************************************************/
-gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
+static gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 {
 	/**current/last item in clipboard  */
 	static gchar *ptext=NULL;
@@ -511,70 +509,46 @@ done:
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void update_clipboards(gchar *intext, gint mode)
+static void update_clipboards(gchar *intext, gint mode)
 {
 	/*g_printf("upclips\n"); */
 	update_clipboard(primary, intext, mode);
 	update_clipboard(clipboard, intext, mode);
 }
 
-static void clear_all ( gpointer user_data);
 /***************************************************************************/
 /** Run a command. For now, just start and stop
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void do_command(gchar *buf, gint len)
+static void do_command(gchar *buf, gint len)
 {
-	/**clean whitepsace  */
-	gchar *p, *news=strdup(buf); 
-	p=g_strchomp(news);
-	int saveme=0;
-  g_fprintf(stderr,"Got '%s' cmd\n",p);
-	if(0 == strncmp(p,"save_historyN ", 14)) 
-		saveme=14;
-	else if(0 == strncmp(p,"save_history ", 13))
-		saveme=13;
-	if(saveme){
-		gchar *tmp=strdup(&p[saveme]);
-		gchar *dir=dirname(tmp);
-		if(0 == access(dir,W_OK)){
-			save_history_as_text(&p[saveme],13==saveme?0:1);
-		}
-		g_free(tmp);
-		goto end;
-	}
-		
-	if(!p_strcmp(p,FIFCMD_RUN_ALL)) {
+  g_printf("Got '%s' cmd\n",buf);
+	if(!p_strcmp(buf,FIFCMD_RUN_ALL)) {
 		cmd_mode|=CMODE_ALL;
-		goto end;
+		return;
 	}	
-	if(!p_strcmp(p,FIFCMD_RUN_CLI)) {
+	if(!p_strcmp(buf,FIFCMD_RUN_CLI)) {
 		cmd_mode|=CMODE_CLI;
-		goto end;
+		return;
 	}	
-	if(!p_strcmp(p,FIFCMD_RUN_PRI)) {
+	if(!p_strcmp(buf,FIFCMD_RUN_PRI)) {
 		cmd_mode|=CMODE_PRI;
-		goto end;
+		return;
 	}	
-	if(!p_strcmp(p,FIFCMD_STOP_ALL)) {
+	if(!p_strcmp(buf,FIFCMD_STOP_ALL)) {
 		cmd_mode&=~(CMODE_ALL);
-		goto end;
+		return;
 	}
-	if(!p_strcmp(p,FIFCMD_STOP_CLI)) {
+		if(!p_strcmp(buf,FIFCMD_STOP_CLI)) {
 		cmd_mode&=~(CMODE_CLI);
-		goto end;
+		return;
 	}
-	if(!p_strcmp(p,FIFCMD_STOP_PRI)) {
+		if(!p_strcmp(buf,FIFCMD_STOP_PRI)) {
 		cmd_mode&=~(CMODE_PRI);
-		goto end;
+		return;
 	}
-	if(!p_strcmp(p,FIFCMD_CLEAR_ALL)) {
-		clear_all((gpointer) NULL);
-		goto end;
-	}
-end:
-	free(news);
+	
 }
 
 /***************************************************************************/
@@ -582,17 +556,16 @@ end:
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void check_clipboards(gint mode)
+static void check_clipboards(gint mode)
 {
 	gchar *ptext, *ctext, *last;
-	int n=0;
 	
 	/*g_printf("check_clipboards\n"); */
 	if(fifo->clen){/**we have a command to execute  */
 			/*fifo->which should be ID_CMD: */
-		if(fifo->dbg) g_fprintf(stderr,"Running CMD '%s'\n",fifo->cbuf);
+		if(fifo->dbg) g_printf("Running CMD '%s'\n",fifo->cbuf);
 		do_command(fifo->cbuf, fifo->clen);
-		if(fifo->dbg) g_fprintf(stderr,"mode is 0x%X\n",cmd_mode);
+		if(fifo->dbg) g_printf("mode is 0x%X\n",cmd_mode);
 		fifo->clen=0;
 		return;
 	}	
@@ -602,22 +575,20 @@ void check_clipboards(gint mode)
 		switch(fifo->which){
 			case ID_PRIMARY:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
-				if(fifo->dbg) g_fprintf(stderr,"Setting PRI '%s'\n",fifo->buf);
+				if(fifo->dbg) g_printf("Setting PRI '%s'\n",fifo->buf);
 				update_clipboard(primary, fifo->buf, H_MODE_NEW);
 				fifo->rlen=0;
-				n=1;
 				break;
 			case ID_CLIPBOARD:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
-				if(fifo->dbg) g_fprintf(stderr,"Setting CLI '%s'\n",fifo->buf);
+				if(fifo->dbg) g_printf("Setting CLI '%s'\n",fifo->buf);
 				update_clipboard(clipboard, fifo->buf, H_MODE_NEW);
-				n=2;
 				fifo->rlen=0;
 				break;
 			
 			default:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
-				/*g_fprintf(stderr,"CLIP not set, discarding '%s'\n",fifo->buf); */
+				g_printf("CLIP not set, discarding '%s'\n",fifo->buf);
 				fifo->rlen=0;
 				break;
 		}
@@ -651,20 +622,18 @@ done:
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean check_for_appindictor( gpointer data)
+static gboolean check_for_appindictor( gpointer data)
 { 
 	int mode=PROC_MODE_STRSTR;
 	if(NULL != appindicator_process && !have_appindicator ){
 		/*g_printf("Looking for '%s'\n",appindicator_process); */
-		if(get_pref_int32("multi_user"))
+		if(get_pref_int32("multi_user")){
 			mode|=PROC_MODE_USER_QUALIFY;
-		
-		if(get_pref_int32("on_unity") || proc_find(appindicator_process,mode,NULL) >0 ){
-			have_appindicator=1;
-			if(NULL == indicator && show_icon)
-				create_app_indicator();	
-			return FALSE;
 		}
+		have_appindicator=1;
+		if(NULL == indicator && show_icon)
+			create_app_indicator();	
+		return FALSE;
 	}
 	if (1 == (int)data)/*short timer, don't call again, bug fix 118*/
 		return FALSE;
@@ -676,7 +645,7 @@ gboolean check_for_appindictor( gpointer data)
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean check_clipboards_tic(gpointer data)
+static gboolean check_clipboards_tic(gpointer data)
 {
 	/** gchar *txt=gtk_clipboard_wait_for_text(clipboard);
 	g_printf("%s\n",txt);*/
@@ -692,6 +661,29 @@ gboolean check_clipboards_tic(gpointer data)
 #endif
 	return TRUE;
 }
+#if 0
+/* Thread function called for each action performed */
+static void *execute_action(void *command)
+{
+  /* Execute action */
+  actions_lock = TRUE;
+  if (!have_appindicator && show_icon) {
+  gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_EXECUTE);
+  gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Executing action..."));
+  }
+  if(system((gchar*)command))
+  	g_fprintf(stderr,"sytem command '%s' failed\n",(gchar *)command);
+  if (!have_appindicator &&show_icon) {
+	gtk_status_icon_set_from_icon_name((GtkStatusIcon*)status_icon, get_pref_string("icon_name"));
+  gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
+  }
+  actions_lock = FALSE;
+  g_free((gchar*)command);
+  /* Exit this thread */
+  pthread_exit(NULL);
+	return NULL;
+}
+#endif
 /* Called when execution action exits */
 static void action_exit(GPid pid, gint status, gpointer data)
 {
@@ -714,24 +706,20 @@ static void action_selected(GtkButton *button, gpointer user_data)
   }
   /* Insert clipboard into command (user_data), and prepare it for execution */
   gchar* clipboard_text = gtk_clipboard_wait_for_text(clipboard);
-	/*g_fprintf(stderr,"Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);   */
 	gchar* quoted_clipboard_text = g_shell_quote(clipboard_text);
 	gchar* command=g_strdup_printf((gchar *)user_data,quoted_clipboard_text);
-	//g_fprintf(stderr," '%s'->",command);
   g_free(clipboard_text);
   g_free(quoted_clipboard_text);
   g_free(user_data);
   gchar* shell_command = g_shell_quote(command);
   g_free(command);
-	
   gchar* cmd = g_strconcat("/bin/sh -c ", shell_command, NULL);
   g_free(shell_command);
-  //g_fprintf(stderr," '%s'\n",cmd);fflush(NULL);  
+  
   /* Execute action */
   GPid pid;
   gchar **argv;
   g_shell_parse_argv(cmd, NULL, &argv, NULL);
-	/*g_fprintf(stderr,"cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);   */
   g_free(cmd);
   g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
   g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
@@ -780,7 +768,7 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
 				return;
 			}
 			current_clipboard_text=p_strdup(c->text);
-			/*g_fprintf(stderr,"Got Text frmo wi.index ele %p '%s'",element,c->text);   */
+			g_fprintf(stderr,"Got Text frmo wi.index ele %p '%s'",element,c->text);  
 			
 		}	else{
 			h->wi.tmp1=0;
@@ -795,7 +783,7 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
 		}
     
     if (current_clipboard_text != NULL)   {
-		/*g_fprintf(stderr,"Got '%s' for edit text\n",current_clipboard_text);  */
+		g_fprintf(stderr,"Got '%s' for edit text\n",current_clipboard_text); 
 			TRACE(g_fprintf(stderr,"Got '%s'\n",current_clipboard_text));
       gtk_text_buffer_set_text(text_buffer, current_clipboard_text, -1);
     }	 else	 {
@@ -906,7 +894,7 @@ finish:
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean history_item_right_click_on_edit(GtkWidget *menuitem, gpointer data)
+static gboolean history_item_right_click_on_edit(GtkWidget *menuitem, gpointer data)
 {
 	struct history_info *h=(struct history_info*)data;
 	h->wi.tmp1|=EDIT_MODE_USE_RIGHT_CLICK;
@@ -919,7 +907,7 @@ gboolean history_item_right_click_on_edit(GtkWidget *menuitem, gpointer data)
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean history_item_right_click_on_copy_all(GtkWidget *menuitem, gpointer data)
+static gboolean history_item_right_click_on_copy_all(GtkWidget *menuitem, gpointer data)
 {
 	GList *element;
 	gchar*str=NULL;
@@ -959,36 +947,23 @@ gboolean history_item_right_click_on_copy_all(GtkWidget *menuitem, gpointer data
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean  handle_history_item_right_click (int i, gpointer data)
+static gboolean handle_history_item_right_click (int i, gpointer data)
 {
   /* we passed the view as userdata when we connected the signal */
 	struct history_info *h=(struct history_info*)data;
-	struct history_item *c=NULL;
-	if(NULL !=h ){
-		GList* element = g_list_nth(history_list, h->wi.index);
-		if(NULL !=element){
-			c=(struct history_item *)(element->data);
-			/*g_printf("%s ",c->text); */
-		}
+	if(NULL !=h && i == HIST_MOVE_TO_OK){
+		handle_marking(h,h->wi.item,h->wi.index,OPERATE_PERSIST);
 	}
-	switch(i){
-		case HIST_MOVE_TO_CANCEL:
-/*			g_print("canceled\n"); */
-			break;
-		case HIST_MOVE_TO_OK:
-/*			g_printf("Move to"); */
-			handle_marking(h,h->wi.item,h->wi.index,OPERATE_PERSIST);
-			break;
-	}
-	/*gtk_widget_grab_focus(h->menu); */
 	return TRUE;
 }
+
 /**callback wrappers for the above function  */
-gboolean  history_item_right_click_on_move (GtkWidget *menuitem, gpointer data)
+static gboolean history_item_right_click_on_move (GtkWidget *menuitem, gpointer data)
 {
 	return handle_history_item_right_click(HIST_MOVE_TO_OK,data);
 }
-gboolean history_item_right_click_on_cancel (GtkWidget *menuitem, gpointer data)
+
+static gboolean history_item_right_click_on_cancel (GtkWidget *menuitem, gpointer data)
 {
 	return handle_history_item_right_click(HIST_MOVE_TO_CANCEL,data);
 }
@@ -998,11 +973,12 @@ gboolean history_item_right_click_on_cancel (GtkWidget *menuitem, gpointer data)
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void  destroy_right_click_history_cb(GtkWidget *attach_widget, GtkMenu *menu)
+static void destroy_right_click_history_cb(GtkWidget *attach_widget, GtkMenu *menu)
 {
 	/*g_printf("%s:\n",__func__); */
 	gtk_widget_destroy	((GtkWidget *) attach_widget);
 }
+
 /***************************************************************************/
 /** Display the right-click menu. h->menu contains the top-level history window
 \n\b Arguments:
@@ -1011,7 +987,7 @@ if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3)
 	view_popup_menu(treeview, event, userdata);
 	h->wi.index contains the element of the item clicked on.
 ****************************************************************************/
-void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint index)
+static void history_item_right_click (struct history_info *h, GdkEventKey *e, gint index)
 {
   GtkWidget *menu, *menuitem;
   
@@ -1072,30 +1048,10 @@ void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint ind
 	
 }
 
-/***************************************************************************/
-/** Clear history, without confirmation.
-\n\b Arguments:
-\n\b Returns:
-****************************************************************************/
-static void clear_all ( gpointer user_data)
-{
-	if( NULL != user_data ){ 
-		struct history_info *h=(struct history_info *)user_data;
-		/* Clear history and free history-related variables */
-		remove_deleted_items(h); /**fix bug 92, Shift/ctrl right-click followed by clear segfaults/double free.  */	
-	}	
-	clear_history();
-	/*g_printf("Clear hist done, h=%p, h->delete_list=%p\n",h, h->delete_list); */
-	update_clipboard(primary, "", H_MODE_INIT);
-  update_clipboard(clipboard, "", H_MODE_INIT);	
-}
-
 /* Called when Clear is selected from history menu */
 static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
 {
 	int clear=1;
-	if(get_pref_int32("disable_clear"))
-		return;
   /* Check for confirm clear option */
   if (get_pref_int32("confirm_clear"))  {
     GtkWidget* confirm_dialog = gtk_message_dialog_new(NULL,
@@ -1110,7 +1066,13 @@ static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
 		gtk_widget_destroy(confirm_dialog);
 	}	
 	if(clear){
-		clear_all(user_data);
+		struct history_info *h=(struct history_info *)user_data;
+		/* Clear history and free history-related variables */
+		remove_deleted_items(h); /**fix bug 92, Shift/ctrl right-click followed by clear segfaults/double free.  */	
+		clear_history();
+		/*g_printf("Clear hist done, h=%p, h->delete_list=%p\n",h, h->delete_list); */
+		update_clipboard(primary, "", H_MODE_INIT);
+    update_clipboard(clipboard, "", H_MODE_INIT);
 	}
       
 }
@@ -1250,51 +1212,84 @@ static gboolean show_actions_menu(gpointer data)
   gtk_menu_shell_append((GtkMenuShell*)menu, gtk_separator_menu_item_new());
   /* Actions */
   gchar* path = g_build_filename(g_get_user_data_dir(), ACTIONS_FILE, NULL);
-	g_fprintf(stderr,"got path '%s'\n",path); fflush(NULL);
   FILE* actions_file = fopen(path, "rb");
   g_free(path);
-  /* Check that it opened and begin read */
-  if (actions_file)
-  {
-    gint size;
-    if(0==fread(&size, 4, 1, actions_file))
-    	g_fprintf(stderr,"1:got 0 items from fread\n");
-    /* Check if actions file is empty */
-    if (!size)
-    {
-      /* File contained no actions so adding empty */
-      menu_item = gtk_menu_item_new_with_label(_("Empty"));
-      gtk_widget_set_sensitive(menu_item, FALSE);
-      gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-    }
-    /* Continue reading items until size is 0 */
-    while (size)
-    {
-      /* Read name */
-      gchar* name = (gchar*)g_malloc(size + 1);
-      if( 0 ==fread(name, size, 1, actions_file))
-      	g_fprintf(stderr,"2:got 0 items from fread\n");
-      name[size] = '\0';
-      menu_item = gtk_menu_item_new_with_label(name);
-      
-      if(0 ==fread(&size, 4, 1, actions_file))
-      	g_fprintf(stderr,"3:got 0 items from fread\n");
-      /* Read command */
-      gchar* command = (gchar*)g_malloc(size + 1);
-      if(0 ==fread(command, size, 1, actions_file))
-      	g_fprintf(stderr,"4:got 0 items from fread\n");
-      command[size] = '\0';
-		  g_print("name='%s' cmd='%s'\n",name,command);
-      if(0 ==fread(&size, 4, 1, actions_file))
-      	g_fprintf(stderr,"5:got 0 items from fread\n");
-      /* Append the action */
-      gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-      g_signal_connect((GObject*)menu_item,        "activate",
-                       (GCallback)action_selected, (gpointer)command);      
-		  g_free(name);
-    }
-    fclose(actions_file);
-  }
+
+	/* Check that it opened and begin read */
+	if (actions_file)
+	{
+		gint size;
+		if(1 != fread(&size, 4, 1, actions_file)){
+			size = 0;
+		}
+
+		/* Check if actions file is empty */
+		if (!size){
+			/* File contained no actions so adding empty */
+			menu_item = gtk_menu_item_new_with_label(_("Empty"));
+			gtk_widget_set_sensitive(menu_item, FALSE);
+			gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+		}else{
+			gchar* name;
+			gchar* command;
+
+			/* Continue reading items until size is 0 */
+			while (size > 0){
+				name = NULL;
+				command = NULL;
+				menu_item = NULL;
+
+				/* Read name */
+				name = (gchar*)g_malloc(size + 1);
+				if(1 != fread(name, size, 1, actions_file)){
+					break;
+				}
+				name[size] = '\0';
+				menu_item = gtk_menu_item_new_with_label(name);
+
+				if(1 != fread(&size, 4, 1, actions_file)){
+					break;
+				}
+
+				/* Read command */
+				command = (gchar*)g_malloc(size + 1);
+				if(1 != fread(command, size, 1, actions_file)){
+					break;
+				}
+				command[size] = '\0';
+				
+				if(1 != fread(&size, 4, 1, actions_file)){
+					break;
+				}
+
+				/* Append the action */
+				gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+				g_signal_connect(
+					(GObject*)menu_item,
+					"activate",
+					(GCallback)action_selected,
+					(gpointer)command
+				);
+
+				command = NULL;
+				menu_item = NULL;
+
+				g_free(name);
+				name = NULL;
+			}
+
+			if (name)
+				g_free(name);
+
+			if (command)
+				g_free(command);
+
+			if (menu_item)
+				g_free(menu_item);
+		}
+
+		fclose(actions_file);
+	}
   else
   {
     /* File did not open so adding empty */
@@ -1332,11 +1327,13 @@ static gboolean selection_done(GtkMenuShell *menushell, gpointer user_data)
 	}	
 	/*g_print("selection_active=%d\n",selection_active); */
 	/*g_print("Got selection_done\n"); */
-	if(h->change_flag && get_pref_int32("save_history")){
-		save_history();
-		h->change_flag=0;
+	if (NULL != h){
+		if(h->change_flag && get_pref_int32("save_history")){
+			save_history();
+			h->change_flag=0;
+		}
 	}
-		  
+
 done:
 	/*gtk_widget_destroy((GtkWidget *)menushell); - fixes annoying GTK_IS_WIDGET/GTK_IS_WINDOW
 	  warnings from GTK when history dialog is destroyed. */
@@ -1348,7 +1345,7 @@ done:
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void set_widget_bg(gchar *color, GtkWidget *w)
+static void set_widget_bg(gchar *color, GtkWidget *w)
 {
   GdkColor c, *cp;
   /**GtkRcStyle *st;
@@ -1440,15 +1437,15 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 	if(0&& NULL != e ){
 		if(GDK_MOTION_NOTIFY==e->type)
 			return FALSE;
-    g_fprintf(stderr,"krc (%x) S%x T%x C%x,SE%x, G%x, W%p, wdg%p",
+    printf("krc (%x) S%x T%x C%x,SE%x, G%x, W%p, wdg%p",
 		e->keyval,e->state,e->type,
 		e->hardware_keycode,e->send_event,e->group,e->window,w);
 		if(GDK_DRAG_ENTER == e->type || GDK_DRAG_LEAVE==e->type){
-			g_fprintf(stderr," Drag %s\n",GDK_DRAG_ENTER == e->type?"ENTER":"LEAVE");
+			printf(" Drag %s\n",GDK_DRAG_ENTER == e->type?"ENTER":"LEAVE");
 		}
 		if(GDK_BUTTON_RELEASE==e->type || GDK_BUTTON_PRESS==e->type  ){
 			GdkEventButton *b=(GdkEventButton *)e;
-			g_fprintf(stderr," button %d State 0x%x\n",b->button, b->state);
+			printf(" button %d State 0x%x\n",b->button, b->state);
 			if(GDK_BUTTON_RELEASE==e->type && 3 == b->button){
 				/*toggle-size-request", */
 				/*allow item_selected to get called  */
@@ -1456,7 +1453,7 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 			}
 			/*return TRUE; */
 		}
-		g_fprintf(stderr,"\n");
+		printf("\n");
 		fflush(NULL);
   }
 	
@@ -1468,11 +1465,11 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 		idx=0;
 		return FALSE;
 	}else if(NULL == kstr){
-		g_fprintf(stderr,"kstr null. Not init\n");
+		g_print("kstr null. Not init\n");
 		return FALSE;
 	}
   if(NULL == e){
-    g_fprintf(stderr,"No Event!\n");
+    g_print("No Event!\n");
     return FALSE;
   }
 		
@@ -1544,9 +1541,10 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 		TRACE(g_fprintf(stderr,"Ignoring key '%c' 0x%02x\n",e->keyval,e->keyval));	
 		return FALSE;
 	}
-  if(e->keyval >= 0xff50 && e->keyval <= 0xff57) /**arrow keys, home,end,pgup,pgdwn  */
-  	return FALSE;
-  	
+	if(e->keyval >= 0xff50 && e->keyval <= 0xff57){ /**arrow keys, home,end,pgup,pgdwn  */
+		return FALSE;
+	}
+
 	if(idx>=KBUF_SIZE){
 		TRACE(g_fprintf(stderr,"keys full\n"));
 		return TRUE;
@@ -1612,111 +1610,80 @@ foundit:
 	TRACE(g_fprintf(stderr,"\n"));
 	return TRUE;
 }	
-extern char **environ;
-/***************************************************************************/
-/** Get environment vars
-\n\b Arguments:
-\n\b Returns:
-****************************************************************************/
-static gchar **get_environment( char *txt )
-{
-	gint i,l;
-	char **env;
-	for (l=0; NULL != environ[l]; ++l);
-	l+=2;/* allow for NULL and our env */
-	if(NULL == (env=malloc(l*sizeof(gchar *))))
-		return NULL;
-	for (i=0; i<l-2; ++i)
-		env[i]=environ[i];
-	env[i++]=txt;
-	env[i]=NULL;
-	return env;
-}
 
 /***************************************************************************/
 /** Set clipboard from history list.
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void set_clipboard_text(struct history_info *h, GList *element)
+static void set_clipboard_text(struct history_info *h, GList *element)
 {
 	int auto_whatever=0;
 	gchar *action=NULL;
 	gchar *txt=NULL;
 	gchar *cmd=NULL;
-	gchar **env=NULL;
-	
+	gchar *quoted=NULL;
 	if(NULL == find_h_item(h->delete_list,NULL,element)){	/**not in our delete list  */
 		/**make a copy of txt, because it gets freed and re-allocated.  */
 		txt=p_strdup(((struct history_item *)(element->data))->text);
-		DTRACE(g_fprintf(stderr,"set_clip_text %s\n",txt));  
+		DTRACE(g_fprintf(stderr,"set_clip_text %s\n",txt));
 		if(use_copy)
 			update_clipboard(clipboard, txt, H_MODE_LIST);
 		if(use_primary)
-	  	    update_clipboard(primary, txt, H_MODE_LIST);	
-		
+		update_clipboard(primary, txt, H_MODE_LIST);
 		auto_whatever=1;
 	}
-  g_signal_emit_by_name ((gpointer)h->menu,"selection-done");
-	if(0 == auto_whatever)
+
+	g_signal_emit_by_name ((gpointer)h->menu,"selection-done");
+	if(0 == auto_whatever){
 		return;
-	/*g_printf("set_clip_text done\n");  */
-	
-	if (get_pref_int32("automatic_paste")) { /** mousedown 2 */
-		if(get_pref_int32("auto_mouse"))
-			action=g_strdup("mousedown 2 && xdotool mouseup 2'");
-		else if(get_pref_int32("auto_key"))
-			action=g_strdup("key ctrl+v'");
-	
-	
-		if( get_pref_int32("key_input")) {
-			gchar *xtxt;
-			int i, e, sl=strlen(txt);
-			for (i=e=0; i<sl; ++i){
-				if( 0x27 == txt[i] || 0x22 == txt[i])
-					++e;
-			}
-				
-			if(NULL == (xtxt=malloc(sl+e+2)))
-				goto done; /**system is likely about to die...  */
-			for (i=e=0; i<sl; ++i){
-				if(0x27 == txt[i] || 0x22 == txt[i])
-					xtxt[e++]='\\';
-			  xtxt[e++]=txt[i];
-			}
-			xtxt[e]=0;
-			/*action=g_strconcat("type \"",xtxt,"\"'",NULL); */
-			action=g_strconcat("txt=",xtxt,NULL);
-			free(xtxt);
+	}
+
+	if(get_pref_int32("automatic_paste")){
+		if(get_pref_int32("auto_mouse")){
+			action = g_strdup("mousedown 2 && xdotool mouseup 2");
+		}else if(get_pref_int32("auto_key")){
+			action = g_strdup("key --clearmodifiers Shift+Insert");
 		}
-	}	
-		
-	if(NULL == action)
-			goto done;
-	if(NULL == environ)
+	}
+
+	if(get_pref_int32("key_input")){
+		quoted = g_shell_quote(txt);
+		action = g_strconcat("type --delay 1 ", quoted, NULL);
+	}
+
+	if(NULL == action){
 		goto done;
-	env=get_environment(action);
-	if(NULL == env)
-		goto done;
-/*  cmd = g_strconcat("/bin/sh -c 'xdotool ", action, NULL); */
-	cmd = g_strconcat("/bin/sh -c 'xdotool type \"$txt\"'", NULL);
-/*	g_fprintf(stderr,"xdotool:#%s#\ntext:#%s#\n",cmd,txt);  */
-  GPid pid;
-  gchar **argv;
-	
+	}
+
+	cmd = g_strconcat("xdotool ", action, NULL);
+	GPid pid;
+	gchar **argv;
+	GError *error = NULL;
 	g_shell_parse_argv(cmd, NULL, &argv, NULL);
-  g_free(cmd);
-  g_spawn_async(NULL, argv, env, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
-  g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
-  g_strfreev(argv);
-  /**end from clipit 1.4.1 */
+	g_free(cmd);
+	gboolean res = g_spawn_async(
+		NULL,
+		argv,
+		NULL,
+		G_SPAWN_SEARCH_PATH_FROM_ENVP | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+		NULL,
+		NULL,
+		&pid,
+		&error
+	);
+	g_strfreev(argv);
+	if (!res){
+		g_fprintf(stderr, "Error when spawning process: %d - %s\n", error->code, error->message);
+	}
+
 done:
-	if(NULL != env)
-		g_free(env);
 	if(NULL != txt)
 		g_free(txt);
 	if(NULL != action)
 		g_free(action);
+	if(NULL != quoted)
+		g_free(quoted);
 }
 
 
@@ -1728,7 +1695,6 @@ done:
 static gboolean my_item_event (GtkWidget *w,GdkEventKey *e, gpointer user)
 {
 	static struct history_info *h=NULL;
-	GtkWidget *menu=NULL;
 	
 	if(NULL==w && NULL==e){
 		h=(struct history_info *)user;
@@ -1737,12 +1703,11 @@ static gboolean my_item_event (GtkWidget *w,GdkEventKey *e, gpointer user)
 	}
 	if(NULL == h)
 		return FALSE;	
-	menu=h->menu;
 	/**check for enter  */
 	if(GDK_MOTION_NOTIFY == e->type)
 		return FALSE;
 	/*printf("my_item_event: T 0x%x S 0x%x ",e->type,e->state); */
-	if(NULL !=h && GDK_ENTER_NOTIFY ==e->type ){/**add to delete   */
+	if(GDK_ENTER_NOTIFY ==e->type ){/**add to delete   */
 		GdkEventCrossing *enter=(GdkEventCrossing *)e;
 		/*printf("state 0x%x\n",enter->state); */
 		/**use shift and right-click  */
@@ -1751,7 +1716,7 @@ static gboolean my_item_event (GtkWidget *w,GdkEventKey *e, gpointer user)
 	}
 	if(GDK_KEY_PRESS == e->type){
 		/*GdkEventKey *k=	(GdkEventKey *)e; */
-		g_fprintf(stderr,"key press %d (0x%x)\n",e->keyval,e->keyval); 
+		printf("key press %d (0x%x)\n",e->keyval,e->keyval); 
 		fflush(NULL);
 	}
 	if(GDK_BUTTON_RELEASE==e->type){
@@ -1800,7 +1765,7 @@ static void item_selected(GtkMenuItem *menu_item, gpointer user_data)
 	GdkEventKey *k=(GdkEventKey *)gtk_get_current_event();
 	GList* element = g_list_nth(history_list, GPOINTER_TO_INT(user_data));
 	/*g_print ("item_selected '%s' type %x val %x\n",(gchar *)((struct history_item *(element->data))->text),k->type, k->keyval);  */
-	if((0xFF0d == k->keyval || 0xFF8d == k->keyval ) && GDK_KEY_PRESS == k->type){
+	if(0xFF0d == k->keyval && GDK_KEY_PRESS == k->type){
 		set_clipboard_text(h,element);
 	}
 }	
@@ -1810,7 +1775,7 @@ static void item_selected(GtkMenuItem *menu_item, gpointer user_data)
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void write_history_menu_items(GList *list, GtkWidget *menu)
+static void write_history_menu_items(GList *list, GtkWidget *menu)
 {
 	GList *element;
 	if(NULL == list)
@@ -1827,7 +1792,7 @@ space-E290A3 - \u2423
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-GString* convert_string(GString* s)
+static GString* convert_string(GString* s)
 {
 	gchar arrow[4]={0xe2,0x86,0x92,0x00};/**0xe28692 (UTF-8 right-arrow) \\  */
 	gchar pharagraph[4]={0xe2,0x81,0x8b,0x00}; /**utf-8 pharagraph symbol \2192 */
@@ -1857,34 +1822,12 @@ GString* convert_string(GString* s)
 u is history.
 \n\b Returns:
 ****************************************************************************/
-void destroy_history_menu(GtkMenuShell *menu, gpointer u)
+static void destroy_history_menu(GtkMenuShell *menu, gpointer u)
 {
 	/*g_printf("%s:\n",__func__); */
 	selection_done(menu,u);	/**allow deleted items to be deleted.  */
 	gtk_widget_destroy((GtkWidget *)menu);
 }
-
-/***************************************************************************/
-/** 
-****************************************************************************/
-void set_menu_xy(GtkMenu *menu, gint *x, gint *y, gboolean *push, gpointer user_data)
-{
-  GdkDisplay *dis;
-  if (NULL != (dis=gdk_display_get_default())) {
-    gdk_display_get_pointer(dis, NULL,x,y,NULL);
-    if( NULL == user_data)
-    	*y=1; // history
-    else
-      *y-=120; //icon menu
-    *x-=50;
-    if(*x<=0)
-    	*x=1;
-		if(*y<=0)
-			*y=1;
-  }
-
-}
-
 /***************************************************************************/
 /**  Called when status icon is left-clicked or action key hit.
 \n\b Arguments:
@@ -2023,7 +1966,7 @@ static gboolean show_history_menu(gpointer data)
       if ((clipboard_temp) && (p_strcmp(hist_text, clipboard_temp) == 0))
       {
         gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", string->str);
-			  /*if( NULL == bold_text) g_fprintf(stderr,"NulBMKUp:'%s'\n",string->str); */
+			  if( NULL == bold_text) g_fprintf(stderr,"NulBMKUp:'%s'\n",string->str);
         gtk_label_set_markup((GtkLabel*)item_label, bold_text);
         g_free(bold_text);
         h.clip_item=menu_item;
@@ -2033,7 +1976,7 @@ static gboolean show_history_menu(gpointer data)
       else if ((primary_temp) && (p_strcmp(hist_text, primary_temp) == 0))
       {
         gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", string->str);
-			  /*if( NULL == italic_text) g_fprintf(stderr,"NulIMKUp:'%s'\n",string->str); */
+			  if( NULL == italic_text) g_fprintf(stderr,"NulIMKUp:'%s'\n",string->str);
         gtk_label_set_markup((GtkLabel*)item_label, italic_text);
         g_free(italic_text);
         h.clip_item=menu_item;
@@ -2096,10 +2039,7 @@ next_loop:
 	
 	if(get_pref_int32("type_search")){
     /* Edit clipboard */
-		if( get_pref_int32("disable_clear"))
-			h.title_item = gtk_image_menu_item_new_with_label( _("Use Alt-E to edit") );
-		else
-				h.title_item = gtk_image_menu_item_new_with_label( _("Use Alt-E to edit, Alt-C to clear") );
+		h.title_item = gtk_image_menu_item_new_with_label( _("Use Alt-E to edit, Alt-C to clear") );
     menu_image = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
     gtk_image_menu_item_set_image((GtkImageMenuItem*)h.title_item, menu_image);
     gtk_menu_shell_append((GtkMenuShell*)menu, h.title_item);    
@@ -2122,7 +2062,7 @@ next_loop:
 	g_signal_connect(menu,"selection-done",(GCallback)destroy_history_menu,(gpointer)&h);
   /* Popup the menu... */
   gtk_widget_show_all(menu);
-  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, get_pref_int32("history_pos")?postition_history:set_menu_xy, NULL, 1, gtk_get_current_event_time());
+  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, get_pref_int32("history_pos")?postition_history:NULL, NULL, 1, gtk_get_current_event_time());
 	/**set last entry at first -fixes bug 2974614 */
 	if(get_pref_int32("reverse_history") && NULL != h.clip_item)
 		gtk_menu_shell_select_item((GtkMenuShell*)menu,h.clip_item);
@@ -2137,7 +2077,7 @@ next_loop:
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gint figure_histories(void)
+static gint figure_histories(void)
 {
 	gint i;
 	if(get_pref_int32("persistent_history")){ 
@@ -2156,21 +2096,19 @@ gint figure_histories(void)
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void _show_history_menu (GtkMenuItem *m, gpointer data)
+static void _show_history_menu (GtkMenuItem *m, gpointer data)
 {
 	g_timeout_add(POPUP_DELAY, show_history_menu, GINT_TO_POINTER(figure_histories()));
 }
-
 /***************************************************************************/
 /** .
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-GtkWidget *create_parcellite_menu(guint button, guint activate_time)
+static GtkWidget *create_parcellite_menu(guint button, guint activate_time)
 {
   /* Declare some variables */
   GtkWidget *menu, *menu_item;
-  gint x,y;
   
   /* Create the menu */
   menu = gtk_menu_new();
@@ -2206,8 +2144,8 @@ GtkWidget *create_parcellite_menu(guint button, guint activate_time)
   gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
   /* Popup the menu... */
   gtk_widget_show_all(menu);
-  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, set_menu_xy, (gpointer)1, button, activate_time);
-  return menu;
+  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, NULL, NULL, button, activate_time);	
+	return menu;
 }
 
 /* Called when status icon is right-clicked */
@@ -2273,7 +2211,7 @@ void setup_icon( void )
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-void create_app_indicator(void)
+static void create_app_indicator(void)
 {
 	if(NULL == indicator_menu)
 		/* Create the menu */
@@ -2319,7 +2257,7 @@ void menu_hotkey(char *keystring, gpointer user_data)
 
 
 /* Startup calls and initializations */
-static void parcellite_init()
+static void parcellite_init(void)
 {
 	int i;
 /* Create clipboard */
@@ -2383,7 +2321,7 @@ static void parcellite_init()
 which - which fifo we write to.
 \n\b Returns:
 ****************************************************************************/
-void write_stdin(struct p_fifo *fifo, int which)
+static void write_stdin(struct p_fifo *fifo, int which)
 {
   if (!isatty(fileno(stdin)))   {
     GString* piped_string = g_string_new(NULL);
@@ -2410,7 +2348,51 @@ void write_stdin(struct p_fifo *fifo, int which)
 	 		
 	}
 }
-	
+
+void null_handler(const gchar *log_domain,
+				 GLogLevelFlags log_level,
+				 const gchar *message,
+				 gpointer user_data) {
+	/* >/dev/null */
+}
+
+typedef enum LockResult {
+	LOCK_ERROR,
+	LOCK_OURS,
+	LOCK_THEIRS
+} LockResult;
+
+static LockResult try_to_lock(void) {
+	LockResult res;
+	uid_t uid = getuid();
+	const gchar *display = g_getenv("DISPLAY");
+	const gchar *seat = g_getenv("XDG_SEAT");
+	const gchar *cookie = g_getenv("XDG_SESSION_COOKIE");
+	gchar *name = g_strdup_printf(
+		PARCELLITE_PROG_NAME "-%d-%s%s%s.lock",
+		uid,
+		seat ? seat : "",
+		display ? display : "",
+		cookie ? cookie : ""
+	);
+
+	gchar *path = g_build_filename(g_get_tmp_dir(), name, NULL);
+	int fd = open(path, O_CREAT, S_IRUSR | S_IWUSR);
+	if (-1 == fd){
+		g_fprintf(stderr, "Error creating lock file `%s`: %s\n", path, strerror(errno));
+		res = LOCK_ERROR;
+		goto exit;
+	}
+
+	int lock_res = flock(fd, LOCK_NB | LOCK_EX);
+	res = (0 == lock_res) ? LOCK_OURS : LOCK_THEIRS;
+
+exit:
+	g_free(path);
+	g_free(name);
+	return res;
+}
+
 /***************************************************************************/
 	/** .
 	\n\b Arguments:
@@ -2422,11 +2404,13 @@ int main(int argc, char *argv[])
 	int mode;
 	
   bindtextdomain(GETTEXT_PACKAGE, PARCELLITELOCALEDIR);
-  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "uTF-8");
   textdomain(GETTEXT_PACKAGE);
   
   /* Initiate GTK+ */
   gtk_init(&argc, &argv);
+  g_log_set_default_handler(null_handler, NULL);
+
 	/**this just maps to the static struct, prefs do not need to be loaded  */
 	pref_mapper(pref2int_map,PM_INIT); 
 	check_dirs(); /**make sure we set up default RC if it doesn't exist.  */
@@ -2437,21 +2421,25 @@ int main(int argc, char *argv[])
 #endif
   /* Parse options */
 	opts=parse_options(argc, argv);
-  if(NULL == opts)
-   	return 1;
+	if(NULL == opts) {
+		return 1;
+	}
 	if(opts->exit)
 		return 0;
 	mode=PROC_MODE_EXACT;
-	if(get_pref_int32("multi_user"))
-	  mode|=PROC_MODE_USER_QUALIFY;
-	/*g_printf("mode=0x%X\n",mode); */
-	if(proc_find(PARCELLITE_PROG_NAME,mode,NULL)<2)	/**1 for me, and 1 for a running instance  */
+
+	LockResult res;
+	if (!(res = try_to_lock())) {
+		return 2;
+	}
+
+	if(LOCK_OURS == res)
 		mode=PROG_MODE_DAEMON; /**first instance  */
 	else
 		mode=PROG_MODE_CLIENT; /**already running, just access fifos & exit.  */
-	
+
 	/**get options/cmd line not parsed.  */
-	if( NULL != opts->leftovers)g_fprintf(stderr,"%s\n",opts->leftovers);
+	if( NULL != opts->leftovers)g_print("%s\n",opts->leftovers);
 	/**init fifo should set up the fifo and the callback (if we are daemon mode)  */
 		if(opts->primary)	{
 			if(NULL == (fifo=init_fifo(FIFO_MODE_PRI|mode))) return 1;
@@ -2519,7 +2507,7 @@ int main(int argc, char *argv[])
 	}	
   
   /* Init Parcellite */
-  parcellite_init(mode);
+  parcellite_init();
   /*g_printf("Start main loop\n"); */
   /* Run GTK+ loop */
   gtk_main();
