@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2024 by RickyRockRat 
+/* Copyright (C) 2007-2008 by Xyhthyx <xyhthyx@gmail.com>
  *
  * This file is part of Parcellite.
  *
@@ -34,8 +34,6 @@ void setup_icon( void ); /**in main.c  */
 #define DEF_HISTORY_LIMIT     25
 #define DEF_HYPERLINKS_ONLY   FALSE
 #define DEF_CONFIRM_CLEAR     TRUE
-#define DEF_DISABLE_CLEAR			FALSE
-#define DEF_HISTORY_SAVE_FORMAT TRUE
 #define DEF_SINGLE_LINE       TRUE
 #define DEF_REVERSE_HISTORY   FALSE
 #define DEF_ITEM_LENGTH       50
@@ -46,7 +44,6 @@ void setup_icon( void ); /**in main.c  */
 #define DEF_ACTIONS_KEY       "<Ctrl><Alt>A"
 #define DEF_MENU_KEY          "<Ctrl><Alt>P"
 #define DEF_NO_ICON           FALSE
-#define DEF_UNITY_ON          FALSE
 
 /**allow lower nibble to become the number of items of this type  */
 #define PREF_TYPE_TOGGLE 0x10
@@ -140,7 +137,7 @@ struct pref_item myprefs[]={
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="automatic_paste",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Auto Paste",.tip="If checked, will use xdotool to paste wherever the mouse is.\nNOTE! Package xdotool MUST BE INSTALLED for this to work.",.val=0},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="auto_key",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Key",.tip="If checked, will use Ctrl-V paste.",.val=0},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="auto_mouse",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Mouse",.tip="If checked, will use middle mouse to paste.",.val=1},
-	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="key_input",.type=PREF_TYPE_TOGGLE|PREF_TYPE_SINGLE_LINE,.desc="Keyboard Input",.tip="If checked, will emit the history entry via the keyboard entry.",.val=0},
+	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="key_input",.type=PREF_TYPE_TOGGLE,.desc="Keyboard Input",.tip="If checked, will emit the history entry via the keyboard entry.",.val=0},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="restore_empty",.type=PREF_TYPE_TOGGLE,.desc="Restore Empty",.tip="If checked, will restore clipboard entry on application exit.",.val=1},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_HIST,.name="rc_edit",.type=PREF_TYPE_TOGGLE,.desc="Right Click Edit",.tip="If checked, edit the history item clicked on.",.val=0},
   
@@ -153,8 +150,6 @@ struct pref_item myprefs[]={
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_MISC,.name="trim_newline",.type=PREF_TYPE_TOGGLE,.desc="Trim Newlines",.tip="If checked, will replace newlines with spaces."},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_MISC,.name="hyperlinks_only",.type=PREF_TYPE_TOGGLE,.desc="Capture hyperlinks only",.tip=NULL,.val=DEF_HYPERLINKS_ONLY},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_MISC,.name="confirm_clear",.type=PREF_TYPE_TOGGLE,.desc="Confirm before clearing history",.tip=NULL,.val=DEF_CONFIRM_CLEAR},
-	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_MISC,.name="disable_clear",.type=PREF_TYPE_TOGGLE,.desc="Disable clearing history",.tip="If checked, disables Alt-C history clear",.val=DEF_DISABLE_CLEAR},
-	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_MISC,.name="save_history_Lineno",.type=PREF_TYPE_TOGGLE,.desc="Use Line Number prefix when saving history",.tip="If checked, prefixes each line number with NHIST_0000/PHIST_0000",.val=DEF_HISTORY_SAVE_FORMAT},	
 	
 /**Display  add icon here...*/
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sfunc=NULL,.sec=PREF_SEC_DISP,.name=NULL,.type=PREF_TYPE_FRAME,.desc="<b>Items</b>",.tip=NULL,.val=0},
@@ -175,7 +170,6 @@ struct pref_item myprefs[]={
 /**miscellaneous that doesn't fit elswhew  */	
   {.adj=NULL,.cval=NULL,.sig=NULL,.sfunc=NULL,.sec=PREF_SEC_XMISC,.name=NULL,.type=PREF_TYPE_FRAME,.desc="<b>Miscellaneous</b>",.tip=NULL,.val=0},
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_XMISC,.name="multi_user",.type=PREF_TYPE_TOGGLE,.desc="Multiuser",.tip="If checked, enables checking multiple concurrent user logic. Use if several different users are logged in at the same time.",.val=TRUE},
-	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_XMISC,.name="on_unity",.type=PREF_TYPE_TOGGLE,.desc="Running Unity",.tip="If checked, Assumes you are running Unity. Use if your systray icon does not show up.",.val=DEF_UNITY_ON},
 	{.adj=NULL,.cval=PARCELLITE_ICON,.sig=NULL,.sec=PREF_SEC_XMISC,.name="icon_name",.type=PREF_TYPE_ENTRY,.desc="Parcellite Icon Name",.tip="Name of Parcellite icon. If this is mis-typed, icon will not appear.",.val=TRUE},
 #ifdef	DEBUG_UPDATE
 	{.adj=NULL,.cval=NULL,.sig=NULL,.sec=PREF_SEC_XMISC,.name="debug_update",.type=PREF_TYPE_TOGGLE,.desc="DebugUpdate",.tip="If checked, enables debug prints on clipboard update logic. This only takes effect when enabled at start up, and may be disabled at compile time.",.val=FALSE},
@@ -201,16 +195,14 @@ GtkTreeSelection* actions_selection;
 static void check_for_tools_exit(GPid pid, gint status, gpointer data)
 {
 	int flag= GPOINTER_TO_INT(data);
-  g_spawn_close_pid(pid);
+	g_spawn_close_pid(pid);
 	if( WIFEXITED(status) ){
 		if(0 ==WEXITSTATUS(status) )
 			tool_bitfield|=flag;
-		
 	}
 	tool_bitfield_check &= ~(flag);
-	g_fprintf(stderr,"Flag 0x%04x, status %d, EXIT %d STAT %d\n",flag,status,WIFEXITED(status),WEXITSTATUS(status) ); 
-		
 }
+
 /***************************************************************************/
 /** Check for installed tools and set flags accordingly.
 \n\b Arguments:
@@ -732,39 +724,70 @@ void read_preferences(int mode)
 /* Read ~/.parcellite/actions into the treeview */
 static void read_actions()
 {
-  /* Open the file for reading */
-  gchar* path = g_build_filename(g_get_user_data_dir(), ACTIONS_FILE, NULL);
-  FILE* actions_file = fopen(path, "rb");
-  g_free(path);
-  /* Check that it opened and begin read */
-  if (actions_file)
-  {
-    /* Keep a row reference */
-    GtkTreeIter row_iter;
-    /* Read the size of the first item */
-    gint size=0;
-    if(0 ==fread(&size, 4, 1, actions_file)) g_print("P1:0 Items read\n");
-    /* Continue reading until size is 0 */
-    while (size != 0)
-    {
-      /* Read name */
-      gchar* name = (gchar*)g_malloc(size + 1);
-      if(0 ==fread(name, size, 1, actions_file))  g_print("P1:0 Items read\n");
-      name[size] = '\0';
-      if(0 ==fread(&size, 4, 1, actions_file))  g_print("P1:0 Items read\n");
-      /* Read command */
-      gchar* command = (gchar*)g_malloc(size + 1);
-      if(0 ==fread(command, size, 1, actions_file))  g_print("P1:0 Items read\n");
-      command[size] = '\0';
-      if(0 ==fread(&size, 4, 1, actions_file))  g_print("P1:0 Items read\n");
-      /* Append the read action */
-      gtk_list_store_append(actions_list, &row_iter);
-      gtk_list_store_set(actions_list, &row_iter, 0, name, 1, command, -1);
-      g_free(name);
-      g_free(command);
-    }
-    fclose(actions_file);
-  }
+	/* Open the file for reading */
+	gchar* path = g_build_filename(g_get_user_data_dir(), ACTIONS_FILE, NULL);
+	FILE* actions_file = fopen(path, "rb");
+	g_free(path);
+	/* Check that it opened and begin read */
+	if (actions_file)
+	{
+		/* Keep a row reference */
+		GtkTreeIter row_iter;
+		/* Read the size of the first item */
+		gint32 size;
+
+		if (1 == fread(&size, 4, 1, actions_file)){
+			gchar* name;
+			gchar* command;
+
+			/* Continue reading until size is 0 */
+			while (size > 0)
+			{
+				name = NULL;
+				command = NULL;
+
+				/* Read name */
+				name = (gchar*)g_malloc(size + 1);
+				if (1 != fread(name, size, 1, actions_file)){
+					break;
+				}
+				name[size] = '\0';
+
+				if (1 != fread(&size, 4, 1, actions_file)){
+					break;
+				}
+
+				/* Read command */
+				command = (gchar*)g_malloc(size + 1);
+				if(1 != fread(command, size, 1, actions_file)){
+					break;
+				}
+				command[size] = '\0';
+
+				if (1 != fread(&size, 4, 1, actions_file)){
+					break;
+				}
+
+				/* Append the read action */
+				gtk_list_store_append(actions_list, &row_iter);
+				gtk_list_store_set(actions_list, &row_iter, 0, name, 1, command, -1);
+
+				g_free(name);
+				name = NULL;
+				
+				g_free(command);
+				command = NULL;
+			}
+
+			if (name)
+				g_free(name);
+
+			if (command)
+				g_free(command);
+		}
+
+		fclose(actions_file);
+	}
 }
 
 /* Save the actions treeview to ~/.local/share/parcellite/actions */
@@ -1067,7 +1090,7 @@ int add_section(int sec, GtkWidget *parent)
 				packit=myprefs[i].w=gtk_menu_item_new_with_label("");
 				child=gtk_bin_get_child((GtkBin *)packit);
 				gtk_misc_set_padding((GtkMisc *)child,0,0);
-				gtk_label_set_markup ((GtkLabel *)child, "<span size=\"1\"> </span>");
+				gtk_label_set_markup ((GtkLabel *)child, "<span size=\"0\"> </span>");
 				break;
 			
 			default: 
@@ -1108,6 +1131,20 @@ int add_section(int sec, GtkWidget *parent)
 	
 }
 
+static void
+run_response_handler (GtkDialog *dialog,
+                      gint response_id,
+                      gpointer data)
+{
+    if (response_id == GTK_RESPONSE_ACCEPT) {
+        /* Apply and save preferences */
+        apply_preferences();
+        save_preferences();
+        save_actions();
+    }
+    gtk_widget_destroy ((GtkWidget*) dialog);
+}
+
 /* Shows the preferences dialog on the given tab */
 void show_preferences(gint tab)
 {
@@ -1293,17 +1330,11 @@ void show_preferences(gint tab)
   
   /* Read actions */
   read_actions();
-  
+
   /* Run the dialog */
   gtk_widget_show_all(dialog);
   gtk_notebook_set_current_page((GtkNotebook*)notebook, tab);
-  if (gtk_dialog_run((GtkDialog*)dialog) == GTK_RESPONSE_ACCEPT)
-  {
-    /* Apply and save preferences */
-    apply_preferences();
-    save_preferences();
-    save_actions();
-  }
-  gtk_widget_destroy(dialog);
-}
 
+  g_signal_connect(dialog, "response", G_CALLBACK (run_response_handler), NULL);
+  gtk_widget_show(dialog);
+}
